import tensorflow as tf
from tensorflow.keras.layers import LSTM, Bidirectional, Dense, Dropout, Activation, Sequential

class AccentLSTM(tf.keras.Model):
    
    def __init__(self, input_shape):
        super(AccentLSTM, self).__init__()
        self.bidirectional_1 = Bidirectional(LSTM(64), input_shape=input_shape)
        self.dropout_1 = Dropout(0.2)
        self.dense_1 = Dense(40)
        self.activation_1 = Activation(activation='softmax')

    def call(self, inputs):
        output = self.bidirectional_1(inputs)
        output = self.dense(self.dropout_1(output))
        return self.activation_1(output)


DEFAULT_CATEGORIES = [
    '0123456789',
    ' ',
    ',.;:!?()\"[]@#$%^&*_+=«»',
    'ёйцукенгшщзхъфывапролджэячсмитьбюЁЙЦУКЕНГШЩЗХЪФЫВАПРОЛДЖЭЯЧСМИТЬБЮ-\''
]


def tokenize(string, categories=DEFAULT_CATEGORIES):
    token = ''
    tokens = []
    category = None
    for char in string:
        if token:
            if category and char in category:
                token += char
            else:
                tokens.append(token)
                token = char
                category = None
                for cat in categories:
                    if char in cat:
                        category = cat
                        break
        else:
            category = None
            if not category:
                for cat in categories:
                    if char in cat:
                        category = cat
                        break
            token += char
    if token:
        tokens.append(token)
    return tokens



import re
from .constants import ALL_FORMS_FILENAME, REG

all_forms = {}
with open(ALL_FORMS_FILENAME, encoding='utf-8') as all_forms_file:
    for form in all_forms_file:
        if "'" in form:
            form = form.strip('\n')
            unstressed_form = re.sub("['`]", '', form)
            if unstressed_form in all_forms:
                all_forms[unstressed_form].append(form.find("'"))
            else:
                all_forms[unstressed_form] = [form.find("'")]

def parse_the_phrase(text):
    text = text.replace("c", "с")  # latin to cyrillic
    regex1 = "[…:,.?!\n]"
    text = re.sub(regex1, " _ ", text).lower()  # mark beginning of clause
    # get rid of "#%&""()*-[0-9][a-z];=>@[\\]^_{|}\xa0'
    regex2 = "[^а-яё'_ -]"
    text = re.sub(regex2, "", text)
    words = text.split(' ')
    return words

def add_endings(wordlist):
    pluswords = []
    for i, word in enumerate(wordlist):
        if not bool(re.search(REG, word)):
            # won't predict, just return (less then two syllables )
            pluswords.append(word)
        elif i == 0 or wordlist[i - 1] == '_':
            pluswords.append('_' + word)
        else:
            context = wordlist[i - 1].replace("'", "")
            if len(context) < 3:
                ending = context
            else:
                ending = context[-3:]
            plusword = ending + '_' + word
            pluswords.append(plusword)
    return pluswords

def is_in_dictionary(word):
    if word.lower() in all_forms:
        return True
    else:
        return False

def dictionary_stress(word):
    stressed_word = ''
    lword = word.lower()
    if len(all_forms[lword]) == 1:
        stressed_word = word[:all_forms[lword][0]] + "'" + \
            word[all_forms[lword][0]:]
    else:
        return None
    return stressed_word

def is_small(word):
    counter = 0
    for letter in word:
        if letter in 'ёуеыаоэяиюЁУЕЫАОЭЯИЮ':
            counter = counter + 1
    if counter <= 1:
        return True


from pathlib import Path

VOWELS = 'аеиоуэюяыё'
REG = '[{}].*[{}]'.format(VOWELS, VOWELS)
MAXLEN = 40
CHARS = ["'", '-', '_', 'а', 'б', 'в', 'г', 'д', 'е', 'ж', 'з', 'и', 'й', 'к',
         'л', 'м', 'н', 'о', 'п', 'р', 'с', 'т', 'у', 'ф', 'х', 'ц', 'ч', 'ш',
         'щ', 'ъ', 'ы', 'ь', 'э', 'ю', 'я', 'ё']
CHAR_INDICES = dict((c, i) for i, c in enumerate(CHARS))
BASE_DIR = Path(__file__).resolve().parent
MODEL_FILE = BASE_DIR / "text_model.json"
WEIGHTS_FILE = BASE_DIR / "weights.96.hdf5"
ALL_FORMS_FILENAME = BASE_DIR / "add_dictionary.sm"


import tensorflow as tf
from tensorflow.keras import Sequential
from tensorflow.keras.layers import LSTM, Bidirectional, Dense, Dropout, Activation

import numpy as np
import re
from pathlib import Path
from .tokenizer import tokenize
from .constants import MODEL_FILE, WEIGHTS_FILE, MAXLEN, CHAR_INDICES, REG
from .utils import *


class Accent(object):

    def __init__(self):
        self.model_file = MODEL_FILE
        self.weights_file = str(WEIGHTS_FILE)
        with open(self.model_file, 'r') as content_file:
            json_string = content_file.read()

        tf.keras.backend.clear_session()

        self.model = tf.keras.models.model_from_json(json_string)
        self.model.load_weights(self.weights_file)

    def _predict(self, word):
        x = np.zeros((1, MAXLEN, len(CHAR_INDICES)), dtype=np.bool)
        for index, letter in enumerate(word):
            pos = MAXLEN - len(word.replace("'", "")) + index
            x[0, pos, CHAR_INDICES[letter]] = 1
        preds = self.model.predict(x, verbose=0)[0]
        preds = preds.tolist()
        max_value = max(preds)
        index = preds.index(max_value)
        # cut left context "ные_мечты" -> "мечты"
        word = word[word.index('_') + 1:]
        index = len(word) - MAXLEN + index
        if index > len(word) - 1:
            print('no %s-th letter in %s' % (index + 1, word))
        else:
            acc_word = word[:index + 1] + '\'' + word[index + 1:]
            return(acc_word)

    def put_stress(self, text, stress_symbol="'"):
        """This function gets any string as an input and returns the same string
        but only with the predicted stress marks.

        All the formating is preserved using this function.
        """
        words = parse_the_phrase(text)
        tokens = tokenize(text)
        accented_phrase = []
        pluswords = add_endings(words)

        for w in pluswords:
            if not bool(re.search(REG, w)):
                pass
            else:
                accented_phrase.append(self._predict(w))
        final = []

        for token in tokens:
            if is_small(token):
                final.append(token)
            else:
                try:
                    temp = accented_phrase[0].replace("'", '')
                except IndexError:
                    temp = ''
                if temp == token.lower():
                    stress_position = accented_phrase[0].find("'")
                    final.append(token[:stress_position] +
                                    stress_symbol + token[stress_position:])
                    accented_phrase = accented_phrase[1:]
                else:
                    final.append(token)
        final = ''.join(final)
        return final



from setuptools import setup
def readme():
    with open('README.rst', 'r', encoding='utf-8') as f:
        return f.read()

setup(name='russtress',
      version='0.1.4',
      description='Package that helps you to put lexical stress in russian text',
      long_description=readme(),
      url='https://github.com/MashaPo/russtress',
      classifiers=[
        'Development Status :: 4 - Beta',
        'Natural Language :: Russian',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Text Processing :: Linguistic',
        'Intended Audience :: Education'
      ],
      author='Maria Ponomareva, Kirill Milintsevich',
      keywords='nlp russian stress linguistic',
      author_email='ponomarevamawa@gmail.com',
      packages=['russtress'],
      install_requires=[
		  'numpy',
		  'scipy',
		  'tensorflow'
      ],
      include_package_data=True,
      zip_safe=False)
